import os
import shutil
import time
from importlib import resources

import click
import phonenumbers

from .client import Client


@click.group()
@click.option(
    "-a",
    "--authorization",
    help="Key to authorise against SimSMS's servers with.",
)
@click.pass_context
def cli(ctx: click.Context, authorization: str):
    path_db = os.path.join(os.path.expanduser("~"), ".ramadan", "burner")
    file_db = os.path.join(path_db, "sms.db")

    if not os.path.isdir(path_db):
        os.makedirs(path_db)

    if not os.path.isfile(file_db):
        with resources.path("burner.resources", "sms.db") as path:
            shutil.copy(path, file_db)

    ctx.obj = ctx.with_resource(Client(authorization, file_db))


@cli.command()
@click.pass_obj
def countries(client: Client):
    """List all of the available countries and their country codes."""
    for country in client.get_countries():
        click.echo(f"{country.code} = {country.name}")


@cli.command()
@click.pass_obj
def services(client: Client):
    """List all of the available services and their codes."""
    for service in client.get_services():
        click.echo(f"{service.code:<8s} = {service.name}")


@cli.command()
@click.argument("service")
@click.pass_obj
def prices(client: Client, service: str):
    """Find the cheapest prices for a given service."""
    for price in client.get_prices_by_service(service):
        click.echo(
            "[{code}] {name:<16s} = ₽{price}".format(
                code=price.country.code,
                name=price.country.name,
                price=price.price,
            )
        )


@cli.command()
@click.argument("country")
@click.argument("service")
@click.option(
    "-d",
    "--delay",
    default=30,
    type=int,
    help="Time in seconds to wait between checks.",
)
@click.pass_obj
def number(client: Client, country: str, service: str, delay: int):
    """Buy a number for a service in a given country, and await an SMS code from it."""
    _id, number = client.get_number(country, service)

    number_data = phonenumbers.parse(number, country)
    click.echo(f"+{number_data.country_code} {number}")

    while True:
        code = client.get_sms(country, service, _id)

        if code is not None:
            click.echo(f"Code received: {code}")
            break

        click.echo("Waiting for a code...")
        time.sleep(delay)


@cli.command()
@click.pass_obj
def reset(client: Client):
    """Reset the cache with the latest information. Authorisation is needed."""
    client.reset_cache()


def main():
    cli(auto_envvar_prefix="SMS")


if __name__ == "__main__":
    main()



from peewee import (
    Model,
    ForeignKeyField,
    TextField,
    CharField,
    FloatField,
    BooleanField,
    Proxy,
    SqliteDatabase,
)

proxy = Proxy()


class BaseModel(Model):
    class Meta:
        database = proxy


class Country(BaseModel):
    image = TextField()
    name = CharField(unique=True)
    code = CharField(2, unique=True)


class Service(BaseModel):
    image = TextField()
    name = CharField(unique=True)
    code = CharField(5, unique=True)


class Price(BaseModel):
    price = FloatField()
    enabled = BooleanField()
    country = ForeignKeyField(Country)
    service = ForeignKeyField(Service)


def create(database_path: str) -> SqliteDatabase:
    """
    Create a connection to the database and initialise it with the necessary tables.

    Args:
        database_path (str): Where to save the database to.

    Returns:
        The new database connection.
    """
    conn = SqliteDatabase(database_path)
    proxy.initialize(conn)
    conn.create_tables([Country, Price, Service])

    return conn



class BurnerException(Exception):
    """Base exception class."""

    pass


class ScraperException(BurnerException):
    """An error occured while scraping data from the website."""

    pass


class InvalidServiceException(BurnerException):
    """The given service code was invalid."""

    pass


class InvalidCountryException(BurnerException):
    """The given country code was invalid."""

    pass


class APIException(BurnerException):
    """Base exception class for API requests."""


class EmptyAuthorisationException(APIException):
    """No authorisation was given, but it is necessary for this action."""

    pass


class InvalidAuthorisationException(APIException):
    """Authorisation was provided, but it was invalid."""

    pass


class NoFundsException(APIException):
    """The given account is out of funds."""

    pass


class NumbersBusyException(APIException):
    """The numbers are busy, try to get the number again in 30 seconds."""

    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['burner', 'burner.resources']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.1,<5.0.0',
 'click>=8.1.3,<9.0.0',
 'peewee>=3.15.3,<4.0.0',
 'phonenumbers>=8.12.57,<9.0.0',
 'requests>=2.28.1,<3.0.0']

entry_points = \
{'console_scripts': ['burner = burner.__main__:main']}

setup_kwargs = {
    'name': 'burner',
    'version': '0.8.1',
    'description': "Script to interact with SimSMS's website.",
    'long_description': "# Burner\n\n![PyPI](https://img.shields.io/pypi/v/burner) ![PyPI - License](https://img.shields.io/pypi/l/burner) ![PyPI - Python Version](https://img.shields.io/pypi/pyversions/burner)\n\nEasy to use script to determine the cheapest price for [SimSMS](https://simsms.org/) codes.\n\n## Installation\n\n### With Pip\n\n```bash\npip install burner\n```\n\n### With Poetry\n\n```bash\npoetry add git+https://github.com/ramadan8/Burner.git\n```\n\n### Manual\n\n```bash\ngit clone https://github.com/ramadan8/Burner --depth 1\npoetry install\n```\n\n## Usage\n\nUse the following command to find the code for the service you want.\n\n```bash\nburner services\n```\n\nThen use the following command to find the price list for the service.\n\n```bash\nburner prices opt29 # This will get the price list for Telegram.\n```\n\nIf you want to refresh the cache for the prices to a newer version, run the following command.\n\n```bash\nburner --authorization <apikey> reset\n```\n\nIf you'd like to buy a phone number, use the following format. An API key with available funds is necessary for this command.\n\n```bash\nburner -a <apikey> number <countrycode> <servicecode>\n```\n\nFor example, you could type the following to get a Russian [Signal](https://signal.org/) code.\n\n```bash\nburner -a <apikey> number RU opt127\n```\n\nYou can also set your API key with the `SMS_AUTHORIZATION` environment variable.\n\nFor more information simply type `burner --help`.\n",
    'author': 'Aidan',
    'author_email': 'ramadan8@riseup.net',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ramadan8/Burner',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import List, Optional, TypedDict

import requests
from bs4 import BeautifulSoup, Tag

from .exceptions import (
    APIException,
    EmptyAuthorisationException,
    InvalidAuthorisationException,
    NoFundsException,
    ScraperException,
)


class CountryPriceResponse(TypedDict):
    id: str
    enable: str
    country_id: str
    service_opt: str
    price: str
    country_shortname: str


class Session:
    def __init__(self, api_key: str = None):
        self._api_key = api_key

        self._session = requests.Session()

    def request(self, method: str, params: Optional[dict] = None) -> dict:
        """
        Make a request to SimSMS's API.

        Args:
            method (str): The method string for the API.
            params (dict?): Any additional data to pass to the request.

        Returns:
            The JSON response from the API.

        Raises:
            EmptyAuthorisationException: No API key was provided.
            InvalidAuthorisationException: The API key provided is invalid.
            NoFundsException: The API key provided has no money in its balance.
            APIException: An error occured during the API request.
        """
        if not self._api_key:
            raise EmptyAuthorisationException

        params.update(
            {
                "metod": method,  # This spelling mistake is on purpose.
                "apikey": self._api_key,
            }
        )

        resp = self._session.get("https://simsms.org/priemnik.php", params=params)
        resp.raise_for_status()

        # "Коды возвращаемых ошибок" subsection: https://simsms.org/new_theme_api.html
        if resp.text in ("API KEY не найден!", "API KEY не получен!"):
            raise InvalidAuthorisationException

        elif resp.text == "Недостаточно средств!":
            raise NoFundsException

        elif resp.text in (
            "Превышено количество попыток!",
            "Произошла неизвестная ошибка.",
            "Произошла внутренняя ошибка сервера.",
            "Неверный запрос.",
        ):
            raise APIException

        data = resp.json()

        # Mostly rate limits or bans.
        if data["response"] in ("5", "6", "7"):
            raise APIException

        return data

    def _fetch_table_data(self, table_id: int) -> List[dict]:
        """
        Fetch data from the tables listed on the API documentation. This is done as
        there is no officially exposed endpoint to crawl countries or services, only
        tables in the documentation.

        https://simsms.org/new_theme_api.html

        Args:
            table_id (int): The index of the table to scrape information from.

        Returns:
            List of `dict`s containing the row's image, name, and code.

        Raises:
            ScraperException: Failed to scrape the requested information.
        """
        resp = self._session.get("https://simsms.org/new_theme_api.html")
        resp.raise_for_status()

        soup = BeautifulSoup(resp.text, "html.parser")

        tables: List[Tag] = soup.find_all("table")

        if len(tables) == 0 or len(tables) < table_id + 1:
            raise ScraperException("The table with the given ID could not be found.")

        data: List[dict] = []
        rows: List[Tag] = tables[table_id].find_all("tr")

        for row in rows:
            row_data: List[Tag] = row.find_all("td")

            if len(row_data) < 4:
                continue

            # TODO: Maybe return something that _isn't_ a plain old `dict`.
            data.append(
                {
                    # "id": int(row_data[0].get_text().strip()),
                    "image": row_data[1].find("img").get("src").strip(),
                    "name": row_data[2].get_text().strip(),
                    "code": row_data[3].get_text().strip(),
                }
            )

        return data

    def fetch_country_list(self) -> List[dict]:
        """Scrape information from the API's 'country' table."""
        return self._fetch_table_data(0)

    def fetch_service_list(self) -> List[dict]:
        """Scrape information from the API's 'service' table."""
        return self._fetch_table_data(1)

    def fetch_prices_by_country(self, country_code: str) -> List[CountryPriceResponse]:
        """
        Fetches all of the available prices for a specific country. This does not
        require an API key to see.

        Args:
            country_code (str): Alpha-2 country code.

        Returns:
            List of prices directly from the API, including country, service, and price
            information.
        """
        params = {"type": "get_prices_by_country", "country_id": country_code}

        resp = self._session.get("https://simsms.org/reg-sms.api.php", params=params)
        resp.raise_for_status()

        return resp.json()



import os
import re
import time
from typing import List, Optional, Tuple

import peewee
from peewee import SqliteDatabase

from .session import Session

from . import database
from .database import Country, Price, Service
from .exceptions import (
    APIException,
    InvalidCountryException,
    InvalidServiceException,
    NumbersBusyException,
)


def _service_provider(self):
    services: List[Service] = Service.select()

    if len(services) == 0:
        Service.insert_many(self._api.fetch_service_list()).execute()


def _country_provider(self):
    countries: List[Country] = Country.select()

    if len(countries) == 0:
        Country.insert_many(self._api.fetch_country_list()).execute()


def _price_provider(self):
    countries: List[Country] = Country.select()

    for country in countries:
        if Price.select().where(Price.country == country).count() > 0:
            continue

        data: List[dict] = []

        for price in self._api.fetch_prices_by_country(country.code):
            try:
                service = Service.get(Service.code == price["service_opt"])
            except peewee.DoesNotExist:
                continue

            data.append(
                {
                    "price": float(price["price"]),
                    "enabled": price["enable"],
                    "country": country,
                    "service": service,
                }
            )

        Price.insert_many(data).execute()

        time.sleep(0.1)


_cache_providers = {
    "service": _service_provider,
    "country": _country_provider,
    "price": _price_provider,
}


def requires_cache(required_features: List[str]):
    """
    Indicates that certain data is necessary to run the request.

    Args:
        required_features (List[str]): List of features that will be needed to run the
            given function. These are available in the `_cache_providers` variable.
    """
    # TODO: Thread lock?
    def decorator(f):
        def wrapper(self, *args, **kwargs):
            for feature in required_features:
                feature_func = _cache_providers.get(feature)

                if not feature_func:
                    raise Exception("Invalid feature requested.", feature)

                feature_func(self)

            return f(self, *args, **kwargs)

        return wrapper

    return decorator


class Client:
    """
    The client will be used to handle any requests to the API and interactions with the
    database in a  well formatted fashion.

    Args:
        api_key (str?): The user's API key for SimSMS. This is not always necessary,
            especially if the user only wishes to check already cached information
            without purchasing numbers.
        database_path (str?): Where to save the database file to. If no path is given,
            it will be saved as "sms.db" to the current working directory.
    """

    def __init__(
        self, api_key: Optional[str] = None, database_path: Optional[str] = None
    ):
        self._api_key = api_key
        self._database_path: str = database_path if database_path else "sms.db"

        self._api = Session(self._api_key)
        self._database: SqliteDatabase = database.create(self._database_path)

    def __enter__(self):
        return self

    def __exit__(self, *_):
        self._database.close()

    def reset_cache(self) -> None:
        """Deletes the old cache database and re-makes it."""
        self._database.close()

        if os.path.isfile(self._database_path) and self._database_path != ":memory:":
            os.remove(self._database_path)

        self._database: SqliteDatabase = database.create(self._database_path)

        for func in _cache_providers.values():
            func(self)

    @requires_cache(["service", "country", "price"])
    def get_prices_by_service(self, service_code: str) -> List[Price]:
        """
        Find all of the available prices for a given service.

        Args:
            service_code (str): Code for the requested service, as determined from the
                database's 'code' parameter.

        Returns:
            A list of available prices in each country.

        Raises:
            InvalidServiceException: The provided service code was not valid.
        """
        if not re.match(r"^opt\d{1,3}$", service_code):
            raise InvalidServiceException

        service: Optional[Service] = Service.get(Service.code == service_code)

        if not service:
            raise InvalidServiceException

        return Price.select().where(Price.service == service).order_by(Price.price)

    @requires_cache(["country"])
    def get_countries(self) -> List[Country]:
        """
        Get a list of every country from the database. If there are no countries stored
        in the database then they will be inserted from the website.

        Returns:
            A list of countries from the database.
        """
        return Country.select().order_by(Country.code)

    @requires_cache(["service"])
    def get_services(self) -> List[Service]:
        """
        Get a list of every service from the database. If there are no services stored
        in the database then they will be inserted from the website.

        Returns:
            A list of services from the database.
        """
        return Service.select().order_by(Service.name)

    def get_number(self, country_code: str, service_code: str) -> Tuple[int, str]:
        """
        Request a phone number to receive codes from. Keep in mind that the phone code
        for the number is not given, and so a library like `phonenumbers` should be used
        to find the phone code from the alpha-2 code.

        Args:
            country_code (str): Alpha-2 country code.
            service_code (str): Code for the requested service, as determined from the
                database's 'code' parameter.

        Returns:
            The ID of the requested phone number, alongside the phone number itself, as
            a tuple of type `Tuple[int, str]`.

        Raises:
            InvalidCountryException: The provided country code was not valid.
            InvalidServiceException: The provided service code was not valid.
            NumbersBusyException: The requested number is currently busy handling
                another request. Please try again later.
        """
        if not re.match(r"^\w\w$", country_code):
            raise InvalidCountryException

        if not re.match(r"^opt\d{1,3}$", service_code):
            raise InvalidServiceException

        data = self._api.request(
            "get_number",
            {
                "country": country_code,
                "service": service_code,
            },
        )

        # "The numbers are busy, try to get the number again in 30 seconds."
        if data["number"] == "":
            raise NumbersBusyException

        return data["id"], data["number"]

    def get_sms(
        self, country_code: str, service_code: str, number_id: int
    ) -> Optional[str]:
        """
        Try to get the SMS code for a specific number.

        Args:
            country_code (str): Alpha-2 country code.
            service_code (str): Code for the requested service, as determined from the
                database's 'code' parameter.
            number_id (int): Identifying ID for the number we are trying to get an SMS
                code from. This will be received alongside a new phone number from the
                `get_number(...)` method.

        Returns:
            The received verification code, or None if it is not ready.

        Raises:
            InvalidCountryException: The provided country code was not valid.
            InvalidServiceException: The provided service code was not valid.
            APIException: An error occured during the API request.
        """
        if not re.match(r"^\w\w$", country_code):
            raise InvalidCountryException

        if not re.match(r"^opt\d{1,3}$", service_code):
            raise InvalidServiceException

        data = self._api.request(
            "get_sms",
            {
                "id": number_id,
                "country": country_code,
                "service": service_code,
            },
        )

        # Code wasn't received yet.
        if data["response"] == "2":
            return None

        # Code was received.
        elif data["response"] in ("1", "4"):
            return data["sms"]

        # Unknown error occured.
        else:
            raise APIException
